/*
Attraction appliquée à des sprites
---------------------------------

sur le même principe que la répulsion,
on va attirer les sprites p5play qui sont dans le cercle autour d'un point...
la souris ou le doigt par exemple !

la fonction d'attraction s'utilise comme celle de la répulsion,
avec un paramètre en plus.

c'est expliqué en fin de sketch...

Plus d'exemples play :
http://p5play.molleindustria.org/examples/index.html
*/

// une variable Array pour stocker les  exemplaires de fantômes
var ghosts = [];

function setup(){ 

	createCanvas(windowWidth, windowHeight);

	// on ajoute 30 fantômes
	for (var i = 0; i < 30; i++) {
		addGhost(random(0, width), random(0, height));
	};
}
function draw(){
	clear(); // nettoyer l'affichage à chaque cycle

	
	for (var i = 0; i < ghosts.length; i++) {

		var g = ghosts[i];

		// on empêche les fantômes de sortir de l'écran
		if(g.position.x<0){
			g.velocity.x = Math.abs(g.velocity.x); // vitesse positive
		} else if(g.position.x>width){
			g.velocity.x = -Math.abs(g.velocity.x); // vitesse négative
		}
		if(g.position.y<0){
			g.velocity.y = Math.abs(g.velocity.y); // vitesse positive
		} else if(g.position.y>height){
			g.velocity.y = -Math.abs(g.velocity.y); // vitesse négative
		}

		// ATTRACTION du fantôme !
		// dans un rayon de 200 px autour du doigt/souris
		// sans atténuation (1)
		attire(ghosts[i], mouseX, mouseY, 200, 1, 20);

		/*
		la fonction attire() est définie plus bas, facile à réutiliser !
		ah, et touchX et touchY marchent très bien à la souris aussi :)
		*/

	};

	// !!! imortant !!! rafraichir l'affichage de tous les Sprites p5play
	drawSprites();

	// aficher le mode d'emploi !
	text("// Attraction de sprites p5play !", 30, 30);
	// text("Cliquer/toucher pour ajouter un fantôme !", 30, 60);
}

function addGhost(x, y){ // ajoute un fantôme de plus, là où on veut

	// on crée un Sprite, vide
	var ghost = createSprite(x, y, 30, 30); // x, y, L, H

	// définir l'animation à afficher
  	ghost.addAnimation("myanim", "img/ghost_spin0001.png", "img/ghost_spin0003.png");

	// et on oue l'animation
	ghost.changeAnimation("myanim");

	// on peut le retourner (symétrie)
	// ghost.mirrorX(-1);

	// on peut changer sa taille au hasard
	ghost.scale = random(0.25, .5);

	// on peut le tourner si on veut
	ghost.rotation = random(-180, 180);

	// si on veut qu'il se déplace, c'est prévu !
	ghost.velocity.x = random(-1, 1);
	ghost.velocity.y = random(-1, 1);

	// on ajoute le fantôme à la "collection" de fantômes
	ghosts.push(ghost);

	// la fonction addGhost a fabriqué un fantôme,
	// et elle nous y donne accès si on veut
	return ghost;
}

// on peut ajouter des fantômes pour tester les performances
// ça marche aussi au doigt
function  mouseReleased(){
	addGhost(touchX, touchY);
}

/* -------------------
ATTRACTION !

l'idée c'est de créer une zone autour d'un point (le doigt/souris ou un autre point)
dans cette zone les fantômes sont attirés,
hors de cette zone les fantômes vivent leur vie comme avant.

une fonction d'attraction ré-utilisable !

les paramètres à donner :
- une sprite p5play - obtenu via createSprite(), ici ça se passe dans addGhost()
- une coordonnée X du point à partir duquel on attire
- une coordonnée y du point à partir duquel on attire
- le rayon de la zone de répulsion
- le facteur d'atténuation (permet d'adoucir ou durcir la répulsion)
	1 c'est assez réactif, 0.5 très réactif, 4 plus doux par exemple
- la distance minimum : trop près ça ne sert à rien d'attirer, ça crée des bugs d'affichage...
	je n'ai pas trouvé mieux pour l'instant comme solution !
	entre 10 et 30 c'est pas mal.
*/
function attire(sprite, pointX, pointY, ray, attenuation, distmin){

	// d'abord on regarde la distance entre le sprite et le point x/y
	var distance = dist(pointX, pointY, sprite.position.x, sprite.position.y);
	// http://p5js.org/reference/#/p5/dist

	// si le sprite est assez près :
	if(distance < ray && distance > distmin){

		// affichage d'une ligne pour se rendre compte... à virer plus tard !
		line(pointX, pointY, sprite.position.x, sprite.position.y);

		// on détermine le "vecteur" entre la souris et le sprite http://p5js.org/reference/#/p5.Vector
		var vect = createVector(pointX-sprite.position.x, pointY-sprite.position.y);
		// remarque : c'est l'opération inverse de la répulsion !

		/*
		plus on est près plus on doit être attiré,
		ça tombe mal, plus on est prêt plus le vecteur est court...

		il va falloir "inverser" la longueur du vecteur :
		- petite distance devient grand vecteur
		- grande distance devient petit vecteur
		*/

		var longueur = ray/distance;

		// changer la longueur du vecteur et garder sa direction
		vect.setMag(longueur); 

		// on n'a plus qu'à appliquer cette force au sprite !
		sprite.position.x += vect.x/attenuation;
		sprite.position.y += vect.y/attenuation; 			

		/* 
		l'atténuation met un peu de douceur...
		et permet de moduler la réactivité de la répulsion
		*/

		/*
		on aurait aussi pu toucher à la vitesse du sprite via
		g.velocity.x
		g.velocity.y
		mais là il peut reprendre sa vitesse initiale, qu'on n'a pas modifié,
		une fois sorti du "champ" de la souris/doigt !
		*/

	}
}